/***************************************************************
 * Name:      wx017Main.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2025-01-30
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "wx017Main.h"
#include "stipple.xpm"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

    if (format == long_f )
    {
#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-Unicode build");
#else
        wxbuild << _T("-ANSI build");
#endif // wxUSE_UNICODE
    }

    return wxbuild;
}

BEGIN_EVENT_TABLE(wx017Frame, wxFrame)
    EVT_CLOSE(wx017Frame::OnClose)
    EVT_PAINT(wx017Frame::OnPaint)
    EVT_MENU(idMenuQuit, wx017Frame::OnQuit)
    EVT_MENU(idMenuAbout, wx017Frame::OnAbout)
    EVT_MENU(idMenuCaps, wx017Frame::OnCaps)
    EVT_MENU(idMenuJoins, wx017Frame::OnJoins)
    EVT_MENU(idMenuPenStyles, wx017Frame::OnPenStyles)
    EVT_MENU(idMenuBrushStyles, wx017Frame::OnBrushStyles)
    EVT_MENU(idMenuDrawBitmap, wx017Frame::OnDrawBitmap)
    EVT_MENU(idMenuBlitBitmap, wx017Frame::OnBlitBitmap)
    EVT_MENU(idMenuStretchBlitBitmap, wx017Frame::OnStretchBlitBitmap)
    EVT_MENU(idMenuIcono, wx017Frame::OnIcono)
    EVT_MENU(idMenuFuentes, wx017Frame::OnFuentes)
    EVT_MENU(idMenuSVG, wx017Frame::OnSVG)
END_EVENT_TABLE()

wx017Frame::wx017Frame(wxFrame *frame, const wxString& title)
    : wxFrame(frame, -1, title, wxDefaultPosition, wxSize(400,400))
{
    wxImage::AddHandler(new wxPNGHandler);

    SetBackgroundStyle(wxBG_STYLE_PAINT);

    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    wxMenu* fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuQuit, _T("&Salir\tAlt-F4"), _T("Abandona la aplicación"));
    mbar->Append(fileMenu, _("&Fichero"));

    wxMenu* penMenu = new wxMenu(_T(""));
    penMenu->Append(idMenuCaps, _T("Tipos de &caps"), _T("Tipos de caps de pluma"));
    penMenu->Append(idMenuJoins, _T("Tipos de &uniones"), _T("Tipos de joins de pluma"));
    penMenu->Append(idMenuPenStyles, _T("&Estilos de línea"), _T("estilos de línea de pluma"));
    mbar->Append(penMenu, _T("&Plumas"));

    wxMenu* brushMenu = new wxMenu(_T(""));
    brushMenu->Append(idMenuBrushStyles, _T("&Estilos de pincel"), _T("estilos de pincel"));
    mbar->Append(brushMenu, _T("P&inceles"));

    wxMenu* bitmapMenu = new wxMenu(_T(""));
    bitmapMenu->Append(idMenuDrawBitmap, _T("&Draw bitmap"), _T("método DC::DrawBitmap"));
    bitmapMenu->Append(idMenuBlitBitmap, _T("&Blit bitmap"), _T("método DC::Blit"));
    bitmapMenu->Append(idMenuStretchBlitBitmap, _T("&StretchBlit bitmap"), _T("método DC::StretchBlit"));
    mbar->Append(bitmapMenu, _T("&Bitmap"));

    wxMenu* iconoMenu = new wxMenu(_T(""));
    iconoMenu->Append(idMenuIcono, _T("&Draw icono"), _T("método DC::DrawIcon"));
    mbar->Append(iconoMenu, _T("&Icono"));

    wxMenu* fuentesMenu = new wxMenu(_T(""));
    fuentesMenu->Append(idMenuFuentes, _T("&Fuentes"), _T("mostrar texto"));
    mbar->Append(fuentesMenu, _T("&Fuentes"));

    wxMenu* svgMenu = new wxMenu(_T(""));
    svgMenu->Append(idMenuSVG, _T("&SVG"), _T("Generar fichero SVG"));
    mbar->Append(svgMenu, _T("&SVG"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _T("&Sobre\tF1"), _T("Muestra información sobre la aplicación"));
    mbar->Append(helpMenu, _T("&Ayuda"));

    SetMenuBar(mbar);

    // create a status bar with some information about the used wxWidgets version
    CreateStatusBar(2);
    SetStatusText(_T("Hola"),0);
    SetStatusText(wxbuildinfo(short_f), 1);
}


wx017Frame::~wx017Frame() {}

void wx017Frame::OnClose(wxCloseEvent &event) {
    Destroy();
}

void wx017Frame::OnCaps(wxCommandEvent &event) {
    funcion=Caps;
    Refresh();
}

void wx017Frame::OnJoins(wxCommandEvent &event) {
    funcion=Joins;
    Refresh();
}

void wx017Frame::OnPenStyles(wxCommandEvent &event) {
    funcion=PenStyles;
    Refresh();
}

void wx017Frame::OnBrushStyles(wxCommandEvent &event) {
    funcion=BrushStyles;
    Refresh();
}

void wx017Frame::OnDrawBitmap(wxCommandEvent &event) {
    funcion=DrawBitmap;
    Refresh();
}

void wx017Frame::OnBlitBitmap(wxCommandEvent &event) {
    funcion=BlitBitmap;
    Refresh();
}

void wx017Frame::OnStretchBlitBitmap(wxCommandEvent &event) {
    funcion=StretchBlitBitmap;
    Refresh();
}

void wx017Frame::OnIcono(wxCommandEvent &event) {
    funcion=Icono;
    Refresh();
}

void wx017Frame::OnFuentes(wxCommandEvent &event) {
    funcion=Fuentes;
    Refresh();
}

void wx017Frame::OnSVG(wxCommandEvent &event) {
    funcion=SVG;
    Refresh();
}

void wx017Frame::OnPaint(wxPaintEvent& event) {
    wxAutoBufferedPaintDC dc(this);
    dc.Clear();

    switch(funcion) {
    case Caps:
        DibujaCaps(dc);
        break;
    case Joins:
        DibujaJoins(dc);
        break;
    case PenStyles:
        DibujaPenStyles(dc);
        break;
    case BrushStyles:
        DibujaBrushStyles(dc);
        break;
    case DrawBitmap:
        DibujaBitmap(dc);
        break;
    case BlitBitmap:
        Blit(dc);
        break;
    case StretchBlitBitmap:
        StretchBlit(dc);
        break;
    case Icono:
        DibujaIcono(dc);
        break;
    case Fuentes:
        DibujaFuentes(dc);
        break;
    case SVG:
        TrataSVG(dc);
    case Nada:
    default:
        break;
    }
}

void wx017Frame::DibujaCaps(wxDC& dc) {
    wxPen pluma(wxColour(240,240,50), 20);
    wxPen fina(wxColour(0,0,0), 1);

    pluma.SetCap(wxCAP_ROUND);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,20), wxPoint(150,150));
    dc.SetPen(fina);
    dc.DrawLine(wxPoint(20,20), wxPoint(150,150));
    pluma.SetCap(wxCAP_PROJECTING);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(120,20), wxPoint(250,150));
    dc.SetPen(fina);
    dc.DrawLine(wxPoint(120,20), wxPoint(250,150));
    pluma.SetCap(wxCAP_BUTT);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(220,20), wxPoint(350,150));
    dc.SetPen(fina);
    dc.DrawLine(wxPoint(220,20), wxPoint(350,150));
}

void wx017Frame::DibujaJoins(wxDC& dc) {
    wxPen pluma(wxColour(240,50, 240), 20);
    wxPen fina(wxColour(0,0,0), 1);

    dc.SetBrush(*wxTRANSPARENT_BRUSH);
    pluma.SetCap(wxCAP_BUTT);
    pluma.SetJoin(wxJOIN_BEVEL);
    dc.SetPen(pluma);
    dc.DrawRectangle(wxPoint(20,20), wxSize(300,100));
    dc.SetPen(fina);
    dc.DrawRectangle(wxPoint(20,20), wxSize(300,100));
    pluma.SetJoin(wxJOIN_MITER);
    dc.SetPen(pluma);
    dc.DrawRectangle(wxPoint(50,50), wxSize(300,100));
    dc.SetPen(fina);
    dc.DrawRectangle(wxPoint(50,50), wxSize(300,100));
    pluma.SetJoin(wxJOIN_ROUND);
    dc.SetPen(pluma);
    dc.DrawRectangle(wxPoint(80,80), wxSize(300,100));
    dc.SetPen(fina);
    dc.DrawRectangle(wxPoint(80,80), wxSize(300,100));
}

void wx017Frame::DibujaPenStyles(wxDC& dc) {
    wxPen pluma(wxColour(50, 250, 50), 5);
	wxDash dash[] = {0, 4, 0, 2, 4, 4, 4, 2};
	wxBitmap bmp(stipple);

    pluma.SetStyle(wxPENSTYLE_SOLID);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,20), wxPoint(350,20));
    pluma.SetStyle(wxPENSTYLE_DOT);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,30), wxPoint(350,30));
    pluma.SetStyle(wxPENSTYLE_LONG_DASH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,40), wxPoint(350,40));
    pluma.SetStyle(wxPENSTYLE_SHORT_DASH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,50), wxPoint(350,50));
    pluma.SetStyle(wxPENSTYLE_DOT_DASH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,60), wxPoint(350,60));
    pluma.SetStyle(wxPENSTYLE_BDIAGONAL_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,70), wxPoint(350,70));
    pluma.SetStyle(wxPENSTYLE_CROSSDIAG_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,80), wxPoint(350,80));
    pluma.SetStyle(wxPENSTYLE_FDIAGONAL_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,90), wxPoint(350,90));
    pluma.SetStyle(wxPENSTYLE_CROSS_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,100), wxPoint(350,100));
    pluma.SetStyle(wxPENSTYLE_HORIZONTAL_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,110), wxPoint(350,110));
    pluma.SetStyle(wxPENSTYLE_VERTICAL_HATCH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,120), wxPoint(350,120));
    pluma.SetDashes(8, dash);
    pluma.SetStyle(wxPENSTYLE_USER_DASH);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,130), wxPoint(350,130));
    pluma.SetDashes(0, nullptr);

    pluma.SetWidth(7);
    pluma.SetStipple(bmp);
    pluma.SetStyle(wxPENSTYLE_STIPPLE);
    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,140), wxPoint(350,140));
}

void wx017Frame::DibujaBrushStyles(wxDC& dc) {
    wxBrush pincel(wxColour(50, 50, 250));
	wxBitmap bmp(stipple);

    pincel.SetStyle(wxBRUSHSTYLE_SOLID);
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,20), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_TRANSPARENT);
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,45), wxSize(300,20));
    pincel.SetStipple(bmp);
    pincel.SetStyle(wxBRUSHSTYLE_STIPPLE);
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,70), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_BDIAGONAL_HATCH);
    pincel.SetColour("#ff0080");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,95), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_CROSSDIAG_HATCH);
    pincel.SetColour("#80ff00");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,120), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_FDIAGONAL_HATCH);
    pincel.SetColour("#8000ff");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,145), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_CROSS_HATCH);
    pincel.SetColour("#0080ff");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,170), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_HORIZONTAL_HATCH);
    pincel.SetColour("#80ff80");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,195), wxSize(300,20));
    pincel.SetStyle(wxBRUSHSTYLE_VERTICAL_HATCH);
    pincel.SetColour("#800000");
    dc.SetBrush(pincel);
    dc.DrawRectangle(wxPoint(20,220), wxSize(300,20));
}

void wx017Frame::DibujaBitmap(wxDC& dc) {
    wxImage imagen("logo.png");
	wxBitmap bmp = wxBitmap(imagen);

    dc.DrawBitmap(bmp, 30, 30);
}

void wx017Frame::Blit(wxDC& dc) {
    wxImage imagen("logo.png");
	wxBitmap bmp = wxBitmap(imagen);
	wxMemoryDC memDC(&dc);
	memDC.SelectObject(bmp);

    dc.Blit(wxPoint(20,20), bmp.GetSize(), &memDC, wxPoint(0,0),wxOR );
}

void wx017Frame::StretchBlit(wxDC& dc) {
    wxImage imagen("logo.png");
	wxBitmap bmp = wxBitmap(imagen);
	wxMemoryDC memDC(&dc);
	memDC.SelectObject(bmp);

    dc.StretchBlit(wxPoint(20,20), wxSize(200,200), &memDC, wxPoint(0,0), wxSize(100, 100));
    dc.StretchBlit(wxPoint(230,20), wxSize(50,50), &memDC, wxPoint(0,0), bmp.GetSize());
}

void wx017Frame::DibujaIcono(wxDC& dc) {
    wxIcon ic(icono_xpm);
    dc.DrawIcon(ic, wxPoint(20,20));
}

void wx017Frame::DibujaFuentes(wxDC& dc) {
    wxFont fuente1(wxFontInfo(20).Bold().Italic());
    wxFont fuente2(wxFontInfo(45));

    dc.SetFont(fuente1);
    dc.DrawText(_T("Texto de prueba"), wxPoint(20,20));
    dc.SetFont(fuente2);
    dc.DrawRotatedText(_T("Con Clase"), wxPoint(20,220), 45);
    dc.SetFont(fuente1);
    dc.SetBackgroundMode(wxBRUSHSTYLE_SOLID);
    dc.SetTextForeground("#20ff20");
    dc.SetTextBackground("#800505");
    dc.DrawText(_T("Con Clase"), wxPoint(20,250));
}

void wx017Frame::TrataSVG(wxDC& dc) {
    DibujaSVG(dc); // Salida en pantalla
    wxSVGFileDC svgDC("dibujo.svg", 320, 240, 72, "cuadrados");
    DibujaSVG(svgDC); // Salida en fichero
}

void wx017Frame::DibujaSVG(wxDC& dc) {
    wxPen pluma(wxColour(240,240,50), 20);
    wxPen fina(wxColour(0,0,0), 1);

    dc.SetPen(pluma);
    dc.DrawLine(wxPoint(20,20), wxPoint(150,20));
    dc.DrawLine(wxPoint(150,20), wxPoint(150,150));
    dc.DrawLine(wxPoint(150,150), wxPoint(20,150));
    dc.DrawLine(wxPoint(20,150), wxPoint(20,20));
    dc.SetPen(fina);
    dc.DrawLine(wxPoint(20,20), wxPoint(150,20));
    dc.DrawLine(wxPoint(150,20), wxPoint(150,150));
    dc.DrawLine(wxPoint(150,150), wxPoint(20,150));
    dc.DrawLine(wxPoint(20,150), wxPoint(20,20));
}

void wx017Frame::OnQuit(wxCommandEvent &event) {
    Destroy();
}

void wx017Frame::OnAbout(wxCommandEvent &event) {
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Bienvenido a ..."));
}
